/* 
 *  $Id: DLBootstrapCapsule.java,v 1.17 1997/08/31 14:41:01 djw Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants;


/**
 * Stand-in Capsule to initiate DL requests from Channels
 * This class is not a real capsule -- it is never sent/received.
 *
 * @author David Wetherall
 */

final class DLBootstrapCapsule extends Capsule {
  final private static byte[] MID = findMID("ants.DLBootstrapCapsule");
  protected byte[] mid() { return MID; }

  final private static byte[] PID = findPID("ants.DLBootstrapCapsule");
  protected byte[] pid() { return PID; }

  final static int DLRESOURCES = 64; // will limit size of code group
  Method dload;
  Xdr buf;

  /** Send the indicated DL request */
  public boolean evaluate(Node n) {
    boolean repeat = true;

    // place a new method in the cache the first time
    if (dload == null) {

      // channel reuses buf, so we must make a deep copy of the type
      Xdr xdr = new Xdr(buf, buf.index);
      TypeID methodID = xdr.TYPEID();

      dload = new Method(methodID);

      n.capsuleCode.enter(dload); 
      repeat = false;
    }

    // want to enqueue in receive order on the code group
    // but place on the method for now if the code group
    // hasn't been identified; merge when it is.

    // note: this copies the buf before it is reused
    if (dload.group == null)
      dload.enqueue(buf);
    else
      dload.group.enqueue(dload, buf);

    DLRequestCapsule r = new DLRequestCapsule(dload);
    r.prime(this);
    int reverse = Capsule.peekPrevious(buf);

    if (Entity.LOGGING)
      n.log(Entity.L[0], "deserialize: "+ (repeat ? "repeat" : "sending") +
	    " DLRequest "+ dload.methodID +" to " +
	    NodeAddress.toString(reverse));
    
    return n.routeForNode(r, reverse);
  }

  /**
   * Constructor just holds the details.  All the work is moved
   * to evaluate so that DL appears as regular capsule processing.
   */

  DLBootstrapCapsule(Method dload, Xdr buf, int node) {
    this.prime(node, DLRESOURCES); // priveleged version
    this.dload = dload; 
    this.buf = buf;
    saveBuffer = true;
  }
}
