/* 
 *  $Id: DLRequestCapsule.java,v 1.18 1997/08/31 14:41:01 djw Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants;

import java.util.Vector;


/**
 * Demand Load Request capsule and processing
 *
 * @author David Wetherall
 */


final class DLRequestCapsule extends Capsule {
  final private static byte[] MID = findMID("ants.DLRequestCapsule");
  protected byte[] mid() { return MID; }

  final private static byte[] PID = findPID("ants.DLRequestCapsule");
  protected byte[] pid() { return PID; }

  private TypeID target;
  private int parts;
  
  public int length() {
    return super.length() + Xdr.TYPEID + Xdr.INT;
  }
  
  public Xdr serialize() {
    Xdr xdr = super.serialize();
    xdr.PUT(target);
    xdr.PUT(parts);

    return xdr;
  }

  public Xdr deserialize() {
    Xdr xdr = super.deserialize();
    target = xdr.TYPEID();
    parts = xdr.INT();

    return xdr;
  }

  public boolean evaluate(Node n) {
    boolean ret = true;

    Method m = n.capsuleCode.lookup(target);
    if (m != null && m.cl != null) {
      
      if (Entity.LOGGING)
	n.log(Entity.L[0], "DLRequest "+ target
	      +" from "+ NodeAddress.toString(getSrc())
	      +" for parts "+ Integer.toHexString(parts));
      
      CodeGroup g = m.group;

      int idx = 0;
      for (int i = 0; i < g.meths.size(); i++) {
	Method mm = (Method)g.meths.elementAt(i);
	for (int p = 0; p < mm.code.length; p += CodeGroup.CHUNK) {
	  if ((parts | g.part(idx)) !=  parts) continue;
	  
	  int l = mm.code.length - p;
	  if (l > CodeGroup.CHUNK) l = CodeGroup.CHUNK;
	  
	  if (Entity.LOGGING)
	    n.log(Entity.L[0], "DLRequest: sending part "+ 
		  p/CodeGroup.CHUNK +" of "+ mm._cn 
		  +" ("+ Integer.toHexString(g.part(idx)) +")");
	  
	  Capsule dl = new DLResponseCapsule(mm, idx, p, l); 
	  dl.prime(this);
	  boolean sent = n.routeForNode(dl, getPrevious());
	  ret = ret && sent;
	  idx++;
	}
      }

    } else if (Entity.LOGGING)
      n.log(Entity.L[0], "DLRequest: unanswerable "+
	    target +" from "+ NodeAddress.toString(getSrc()));
    
    return ret;
  }
  
  DLRequestCapsule() { }

  DLRequestCapsule(Method m) { 
    target = m.methodID;
    if (m.group != null)
      parts = ~m.group.parts;
    else
      parts = -1;
  }
}

