/* 
 *  $Id: Manager.java,v 1.6 1997/08/08 15:09:14 jrsantos Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants;

import java.util.Date;

import utils.*;


class EntityList {
  Entity entity;
  EntityList next;

  public EntityList(Entity e) {
    this(e, null);
  }

  public EntityList(Entity e, EntityList el) {
    entity = e;
    next = el;
  }

  public int length() {
    int i = 1;
    EntityList c = next;
    while (c != null) { c = c.next; i++; }
    return i;
  }
}


/**
 * Shell for all manager entities
 *
 * @author David Wetherall
 */


public abstract class Manager 
  extends Entity
{
  protected EntityList children;

  public void add(Entity e) {
    children = new EntityList(e, children);
  }

  final protected void log(int level, String msg) {
    if (LOGGING && (logLevel & level) != 0) {
      Date d = new Date();
      System.out.println(d.getHours() + ":" + d.getMinutes() + ":" +
      			 d.getSeconds() + ": " + msg);
      System.out.flush();
    }
  }

  /** No-argument constructor, for calls through newInstance */
  public Manager() { super(); add(this); }

  /** Normal constructor gives manager a name */
  public Manager(String n) { super(n); add(this); }

}
