/* 
 *  $Id: RouteTable.java,v 1.7 1997/08/29 17:22:57 djw Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants;

import java.io.*;
import java.util.Hashtable;
import java.util.Enumeration;


/**
 * routing table
 *
 * @author David Wetherall
 */

public class RouteTable extends Hashtable {
  public int node;

  private Hashtable neighbor_table; //ulana
  public RouteEntry[] neighbor_list;


  private RouteEntry cast(Object o) {
    if (o instanceof RouteEntry) {
      return (RouteEntry)o;
    }
    return null;
  }

  public RouteEntry put(int key, RouteEntry element) {

    neighbor_table.put(new Integer(element.nxt), element);

    return cast(super.put(new Integer(key), element));
  }

  public RouteEntry get(int key) {
    return cast(super.get(new Integer(key)));
  }

  public RouteEntry remove(int key) {
    return cast(super.remove(new Integer(key)));
  }

  public void read(String file) 
    throws Exception
  {
    FileInputStream f = new FileInputStream(file); 
    StreamTokenizer st = new StreamTokenizer(f);
    st.resetSyntax();
    st.whitespaceChars(0, ' ');
    st.wordChars('0', '9');
    st.wordChars('.', '.');
    st.wordChars(':', ':');
    st.wordChars('-', '-');
    st.wordChars('A', 'Z');
    st.wordChars('a', 'z');
    st.commentChar('#');
  
    int src = 0;
    int dst = 0;
    int nxt = 0;
    ChannelAddress addr = null;

    while (st.nextToken() == StreamTokenizer.TT_WORD) {
      if (src == 0) {
	src = NodeAddress.fromString(st.sval);
      }
      else if (dst == 0) {
	dst = NodeAddress.fromString(st.sval);
      } 
      else if (nxt == 0) {
	nxt = NodeAddress.fromString(st.sval);
      }
      else {
	addr = ChannelAddress.New(st.sval);
	if (src == node) {
	  put(dst, new RouteEntry(nxt, addr));
	}
	src = 0;
	dst = 0;
	nxt = 0;
	addr = null;
      }
    }
    
    if (src != 0) 
      throw new IOException("premature end of input");
    else if (st.ttype != StreamTokenizer.TT_EOF)
      throw new IOException("can't handle " + st.toString());


    //ulana
    neighbor_list = new RouteEntry[neighbor_table.size()];

    Enumeration enum = neighbor_table.elements();

    int i = 0;
    while ((enum.hasMoreElements() == true) && (i < neighbor_list.length)) {
      neighbor_list[i] = cast(enum.nextElement());
      i++;
    }
    if (enum.hasMoreElements() == true)
      throw new IOException("neighbor enumeration not finished");
    if (i < neighbor_list.length)
      throw new IOException("neighbor array not filled");
 
  }
  
  public RouteTable(int n) {
    super();
    node = n;

    neighbor_table = new Hashtable(); //ulana
  }

  public static void main(String[] args)
       throws Exception
  {
    if (args.length != 2) {
      System.out.println("usage: RouteTable <node> <routes>");
      System.exit(1);
    }

    RouteTable rt = new RouteTable(NodeAddress.fromString(args[0]));
    rt.read(args[1]);
    System.out.println(rt.toString());
  }
  
}

