/* 
 *  $Id: UDPChannel.java,v 1.17 1997/08/25 21:24:01 djw Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants;

import java.net.*;
import java.io.*;

/**
 * UDP channel for nodes
 *
 * @author David Wetherall
 */

public class UDPChannel
  extends Channel
{
  static final int BUFSIZE = 1500;
  DatagramSocket dsock;
  
  public boolean send(byte[] buf, int length, ChannelAddress dst) {
    if (! (dst instanceof UDPChannelAddress)) {
      Entity.warn("UDPChannel.send() passed non-UDP destination address");
      return false;
    }
    
    UDPChannelAddress a = (UDPChannelAddress)dst;
    DatagramPacket fr = new DatagramPacket(buf, length, a.address, a.port);
    try {
      dsock.send(fr);
      if (LOGGING)
	log(L[7], "transmitted " + fr.getLength() + " bytes");
    } catch (IOException e) {
      // Log("UDPChannel.send datagram send raises " + e);
      return false;
    }
    
    // Log("... done");
    
    return true;
  }
    
  public Capsule receive() throws Exception {
    DatagramPacket p = new DatagramPacket(rxbuf.buf, rxbuf.len);
    Capsule c = null;
    boolean latch = false;
    
    while (!latch) {
      try {
	dsock.receive(p);
	latch = true;
	if (LOGGING)
	  log(L[7], "received " + p.getLength() + " bytes");
      } catch (IOException e) {
	if (LOGGING)
	  log(L[1], "DatagramSocket.receive() caught " + e);
	continue;
      }

      c = deserialize(new Xdr(rxbuf, 0, p.getLength())); 
    }
    return c;
  }  
  
  public UDPChannel(Manager b, String n, UDPChannelAddress src)
    throws IOException
  {
    super(b, n, src);
    dsock = new DatagramSocket(src.port);
    txbuf = new Xdr(BUFSIZE);
    rxbuf = new Xdr(BUFSIZE);
  }
  
  public UDPChannel(Manager b, String n, String hp)
    throws Exception
  {
    this(b, n, new UDPChannelAddress(hp));
  }
  
}
